# Copyright 2020 The StackStorm Authors.
# Copyright 2019 Extreme Networks, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import absolute_import

from st2client.models.reactor import Rule
from st2client.commands import resource
from st2client.formatters import table


class RuleBranch(resource.ResourceBranch):
    def __init__(self, description, app, subparsers, parent_parser=None):
        super(RuleBranch, self).__init__(
            Rule,
            description,
            app,
            subparsers,
            parent_parser=parent_parser,
            commands={
                "list": RuleListCommand,
                "get": RuleGetCommand,
                "update": RuleUpdateCommand,
                "delete": RuleDeleteCommand,
            },
        )

        self.commands["enable"] = RuleEnableCommand(
            self.resource, self.app, self.subparsers
        )
        self.commands["disable"] = RuleDisableCommand(
            self.resource, self.app, self.subparsers
        )


class RuleListCommand(resource.ResourceTableCommand):
    display_attributes = ["ref", "pack", "description", "enabled"]
    display_attributes_iftt = ["ref", "trigger.ref", "action.ref", "enabled"]

    def __init__(self, resource, *args, **kwargs):

        self.default_limit = 50

        super(RuleListCommand, self).__init__(
            resource,
            "list",
            "Get the list of the %s most recent %s."
            % (self.default_limit, resource.get_plural_display_name().lower()),
            *args,
            **kwargs,
        )

        self.resource_name = resource.get_plural_display_name().lower()
        self.group = self.parser.add_argument_group()
        self.parser.add_argument(
            "-n",
            "--last",
            type=int,
            dest="last",
            default=self.default_limit,
            help=(
                "List N most recent %s. Use -n -1 to fetch the full result \
                                       set."
                % self.resource_name
            ),
        )
        self.parser.add_argument(
            "--iftt",
            action="store_true",
            help="Show trigger and action in display list.",
        )
        self.parser.add_argument(
            "-p",
            "--pack",
            type=str,
            help=("Only return resources belonging to the" " provided pack"),
        )
        self.group.add_argument(
            "-c", "--action", help="Action reference to filter the list."
        )
        self.group.add_argument(
            "-g", "--trigger", help="Trigger type reference to filter the list."
        )
        self.enabled_filter_group = self.parser.add_mutually_exclusive_group()
        self.enabled_filter_group.add_argument(
            "--enabled", action="store_true", help="Show rules that are enabled."
        )
        self.enabled_filter_group.add_argument(
            "--disabled", action="store_true", help="Show rules that are disabled."
        )

    @resource.add_auth_token_to_kwargs_from_cli
    def run(self, args, **kwargs):
        # Filtering options
        if args.pack:
            kwargs["pack"] = args.pack
        if args.action:
            kwargs["action"] = args.action
        if args.trigger:
            kwargs["trigger"] = args.trigger
        if args.enabled:
            kwargs["enabled"] = True
        if args.disabled:
            kwargs["enabled"] = False
        if args.iftt:
            # switch attr to display the trigger and action
            args.attr = self.display_attributes_iftt

        include_attributes = self._get_include_attributes(args=args)
        if include_attributes:
            include_attributes = ",".join(include_attributes)
            kwargs["params"] = {"include_attributes": include_attributes}

        return self.manager.query_with_count(limit=args.last, **kwargs)

    def run_and_print(self, args, **kwargs):
        instances, count = self.run(args, **kwargs)
        if args.json or args.yaml:
            self.print_output(
                instances,
                table.MultiColumnTable,
                attributes=args.attr,
                widths=args.width,
                json=args.json,
                yaml=args.yaml,
            )
        else:
            self.print_output(
                instances,
                table.MultiColumnTable,
                attributes=args.attr,
                widths=args.width,
            )

            if args.last and count and count > args.last:
                table.SingleRowTable.note_box(self.resource_name, args.last)


class RuleGetCommand(resource.ContentPackResourceGetCommand):
    display_attributes = ["all"]
    attribute_display_order = [
        "id",
        "uid",
        "ref",
        "pack",
        "name",
        "description",
        "enabled",
    ]


class RuleUpdateCommand(resource.ContentPackResourceUpdateCommand):
    pass


class RuleEnableCommand(resource.ContentPackResourceEnableCommand):
    display_attributes = ["all"]
    attribute_display_order = [
        "id",
        "ref",
        "pack",
        "name",
        "enabled",
        "description",
        "enabled",
    ]


class RuleDisableCommand(resource.ContentPackResourceDisableCommand):
    display_attributes = ["all"]
    attribute_display_order = [
        "id",
        "ref",
        "pack",
        "name",
        "enabled",
        "description",
        "enabled",
    ]


class RuleDeleteCommand(resource.ContentPackResourceDeleteCommand):
    pass



# Copyright 2020 The StackStorm Authors.
# Copyright 2019 Extreme Networks, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import absolute_import

from st2client.models.reactor import Sensor
from st2client.commands import resource


class SensorBranch(resource.ResourceBranch):
    def __init__(self, description, app, subparsers, parent_parser=None):
        super(SensorBranch, self).__init__(
            Sensor,
            description,
            app,
            subparsers,
            parent_parser=parent_parser,
            read_only=True,
            commands={"list": SensorListCommand, "get": SensorGetCommand},
        )

        self.commands["enable"] = SensorEnableCommand(
            self.resource, self.app, self.subparsers
        )
        self.commands["disable"] = SensorDisableCommand(
            self.resource, self.app, self.subparsers
        )


class SensorListCommand(resource.ContentPackResourceListCommand):
    display_attributes = ["ref", "pack", "description", "enabled"]


class SensorGetCommand(resource.ContentPackResourceGetCommand):
    display_attributes = ["all"]
    attribute_display_order = [
        "id",
        "uid",
        "ref",
        "pack",
        "name",
        "enabled",
        "entry_point",
        "artifact_uri",
        "trigger_types",
    ]


class SensorEnableCommand(resource.ContentPackResourceEnableCommand):
    display_attributes = ["all"]
    attribute_display_order = [
        "id",
        "ref",
        "pack",
        "name",
        "enabled",
        "poll_interval",
        "entry_point",
        "artifact_uri",
        "trigger_types",
    ]


class SensorDisableCommand(resource.ContentPackResourceDisableCommand):
    display_attributes = ["all"]
    attribute_display_order = [
        "id",
        "ref",
        "pack",
        "name",
        "enabled",
        "poll_interval",
        "entry_point",
        "artifact_uri",
        "trigger_types",
    ]



# Copyright 2020 The StackStorm Authors.
# Copyright 2019 Extreme Networks, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import absolute_import

import json
import logging

from st2client.commands import resource
from st2client.formatters import table
from st2client.models.inquiry import Inquiry
from st2client.utils.interactive import InteractiveForm

LOG = logging.getLogger(__name__)

DEFAULT_SCOPE = "system"


class InquiryBranch(resource.ResourceBranch):
    def __init__(self, description, app, subparsers, parent_parser=None):

        super(InquiryBranch, self).__init__(
            Inquiry,
            description,
            app,
            subparsers,
            parent_parser=parent_parser,
            read_only=True,
            commands={"list": InquiryListCommand, "get": InquiryGetCommand},
        )

        # Register extended commands
        self.commands["respond"] = InquiryRespondCommand(
            self.resource, self.app, self.subparsers
        )


class InquiryListCommand(resource.ResourceCommand):

    # Omitting "schema" and "response", as it doesn't really show up in a table well.
    # The user can drill into a specific Inquiry to get this
    display_attributes = ["id", "roles", "users", "route", "ttl"]

    def __init__(self, resource, *args, **kwargs):

        self.default_limit = 50

        super(InquiryListCommand, self).__init__(
            resource,
            "list",
            "Get the list of the %s most recent %s."
            % (self.default_limit, resource.get_plural_display_name().lower()),
            *args,
            **kwargs,
        )

        self.resource_name = resource.get_plural_display_name().lower()
        self.parser.add_argument(
            "-n",
            "--last",
            type=int,
            dest="last",
            default=self.default_limit,
            help=(
                "List N most recent %s. Use -n -1 to fetch the full result \
                                       set."
                % self.resource_name
            ),
        )

        # Display options
        self.parser.add_argument(
            "-a",
            "--attr",
            nargs="+",
            default=self.display_attributes,
            help=(
                "List of attributes to include in the "
                'output. "all" will return all '
                "attributes."
            ),
        )
        self.parser.add_argument(
            "-w",
            "--width",
            nargs="+",
            type=int,
            default=None,
            help=("Set the width of columns in output."),
        )

    @resource.add_auth_token_to_kwargs_from_cli
    def run(self, args, **kwargs):
        return self.manager.query_with_count(limit=args.last, **kwargs)

    def run_and_print(self, args, **kwargs):
        instances, count = self.run(args, **kwargs)

        self.print_output(
            reversed(instances),
            table.MultiColumnTable,
            attributes=args.attr,
            widths=args.width,
            json=args.json,
            yaml=args.yaml,
        )
        if args.last and count and count > args.last:
            table.SingleRowTable.note_box(self.resource_name, args.last)


class InquiryGetCommand(resource.ResourceGetCommand):
    pk_argument_name = "id"
    display_attributes = ["id", "roles", "users", "route", "ttl", "schema"]

    def __init__(self, kv_resource, *args, **kwargs):
        super(InquiryGetCommand, self).__init__(kv_resource, *args, **kwargs)


class InquiryRespondCommand(resource.ResourceCommand):
    display_attributes = ["id", "response"]

    def __init__(self, resource, *args, **kwargs):
        super(InquiryRespondCommand, self).__init__(
            resource,
            "respond",
            "Respond to an %s." % resource.get_display_name().lower(),
            *args,
            **kwargs,
        )

        self.parser.add_argument("id", metavar="id", help="Inquiry ID")
        self.parser.add_argument(
            "-r",
            "--response",
            type=str,
            dest="response",
            default=None,
            help=(
                "Entire response payload as JSON string " "(bypass interactive mode)"
            ),
        )

    @resource.add_auth_token_to_kwargs_from_cli
    def run(self, args, **kwargs):
        instance = Inquiry()
        instance.id = args.id
        inquiry = self.get_resource_by_id(id=args.id, **kwargs)
        if args.response:
            instance.response = json.loads(args.response)
        else:
            response = InteractiveForm(
                inquiry.schema.get("properties")
            ).initiate_dialog()
            instance.response = response

        return self.manager.respond(
            inquiry_id=instance.id, inquiry_response=instance.response, **kwargs
        )

    def run_and_print(self, args, **kwargs):
        instance = self.run(args, **kwargs)
        print("\nResponse accepted for inquiry %s." % instance.id)



# Copyright 2020 The StackStorm Authors.
# Copyright 2019 Extreme Networks, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import absolute_import

import abc
import logging

import six


LOG = logging.getLogger(__name__)


class Formatter(six.with_metaclass(abc.ABCMeta, object)):
    @classmethod
    @abc.abstractmethod
    def format(cls, subject, *args, **kwargs):
        """Override this method to customize output format for the subject."""
        raise NotImplementedError



# Copyright 2020 The StackStorm Authors.
# Copyright 2019 Extreme Networks, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import absolute_import

import json
import logging

import orjson
import yaml

from st2client import formatters
from st2client.utils import jsutil

__all__ = ["JsonFormatter", "YAMLFormatter"]

LOG = logging.getLogger(__name__)


class BaseFormatter(formatters.Formatter):
    @classmethod
    def format(self, subject, *args, **kwargs):
        attributes = kwargs.get("attributes", None)
        if type(subject) is str:
            subject = orjson.loads(subject)
        elif not isinstance(subject, (list, tuple)) and not hasattr(
            subject, "__iter__"
        ):
            doc = subject if isinstance(subject, dict) else subject.__dict__
            keys = (
                list(doc.keys())
                if not attributes or "all" in attributes
                else attributes
            )
            docs = jsutil.get_kvps(doc, keys)
        else:
            docs = []
            for item in subject:
                doc = item if isinstance(item, dict) else item.__dict__
                keys = (
                    list(doc.keys())
                    if not attributes or "all" in attributes
                    else attributes
                )
                docs.append(jsutil.get_kvps(doc, keys))

        return docs


class JsonFormatter(BaseFormatter):
    @classmethod
    def format(self, subject, *args, **kwargs):
        docs = BaseFormatter.format(subject, *args, **kwargs)
        return json.dumps(docs, indent=4, sort_keys=True)


class YAMLFormatter(BaseFormatter):
    @classmethod
    def format(self, subject, *args, **kwargs):
        docs = BaseFormatter.format(subject, *args, **kwargs)
        return yaml.safe_dump(docs, indent=4, default_flow_style=False)



# Copyright 2020 The StackStorm Authors.
# Copyright 2019 Extreme Networks, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import absolute_import
import unittest2

from st2client.utils.misc import merge_dicts


class MiscUtilTestCase(unittest2.TestCase):
    def test_merge_dicts(self):
        d1 = {"a": 1}
        d2 = {"a": 2}
        expected = {"a": 2}

        result = merge_dicts(d1, d2)
        self.assertEqual(result, expected)

        d1 = {"a": 1}
        d2 = {"b": 1}
        expected = {"a": 1, "b": 1}

        result = merge_dicts(d1, d2)
        self.assertEqual(result, expected)

        d1 = {"a": 1}
        d2 = {"a": 3, "b": 1}
        expected = {"a": 3, "b": 1}

        result = merge_dicts(d1, d2)
        self.assertEqual(result, expected)

        d1 = {"a": 1, "m": None}
        d2 = {"a": None, "b": 1, "c": None}
        expected = {"a": 1, "b": 1, "c": None, "m": None}

        result = merge_dicts(d1, d2)
        self.assertEqual(result, expected)

        d1 = {"a": 1, "b": {"a": 1, "b": 2, "c": 3}}
        d2 = {"b": {"b": 100}}
        expected = {"a": 1, "b": {"a": 1, "b": 100, "c": 3}}

        result = merge_dicts(d1, d2)
        self.assertEqual(result, expected)



# Copyright 2020 The StackStorm Authors.
# Copyright 2019 Extreme Networks, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import time
import atexit
import platform
import cProfile

__all__ = ["setup_regular_profiler"]


def setup_regular_profiler(service_name: str) -> None:
    """
    Set up regular Python cProf profiler and write result to a file on exit.
    """
    profiler = cProfile.Profile()
    profiler.enable()

    file_path = os.path.join(
        "/tmp", "%s-%s-%s.cprof" % (service_name, platform.machine(), int(time.time()))
    )

    print("Eventlet profiler enabled")
    print("Profiling data will be saved to %s on exit" % (file_path))

    def stop_profiler():
        profiler.disable()
        profiler.dump_stats(file_path)
        print("Profiling data written to %s" % (file_path))
        print("You can view it using: ")
        print("\t python3 -m pstats %s" % (file_path))

    atexit.register(stop_profiler)



#!/usr/bin/env python
# Copyright 2020 The StackStorm Authors.
# Copyright 2019 Extreme Networks, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os.path

from setuptools import setup, find_packages

from dist_utils import fetch_requirements
from dist_utils import apply_vagrant_workaround

from st2client import __version__

ST2_COMPONENT = "st2client-gcx"
BASE_DIR = os.path.dirname(os.path.abspath(__file__))
REQUIREMENTS_FILE = os.path.join(BASE_DIR, "requirements.txt")
README_FILE = os.path.join(BASE_DIR, "README.rst")

install_reqs, dep_links = fetch_requirements(REQUIREMENTS_FILE)
apply_vagrant_workaround()

with open(README_FILE) as f:
    readme = f.read()

setup(
    name=ST2_COMPONENT,
    version=__version__,
    description=(
        "Python client library and CLI for the StackStorm (st2) event-driven "
        "automation platform."
    ),
    long_description=readme,
    long_description_content_type="text/x-rst",
    author="StackStorm",
    author_email="info@stackstorm.com",
    url="https://stackstorm.com/",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Information Technology",
        "Intended Audience :: Developers",
        "Intended Audience :: System Administrators",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: POSIX :: Linux",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
    ],
    install_requires=install_reqs,
    dependency_links=dep_links,
    test_suite=ST2_COMPONENT,
    zip_safe=False,
    include_package_data=True,
    packages=find_packages(exclude=["setuptools", "tests"]),
    entry_points={"console_scripts": ["st2 = st2client.shell:main"]},
    project_urls={
        "Pack Exchange": "https://exchange.stackstorm.org",
        "Repository": "https://github.com/StackStorm/st2",
        "Documentation": "https://docs.stackstorm.com",
        "Community": "https://stackstorm.com/community-signup",
        "Questions": "https://github.com/StackStorm/st2/discussions",
        "Donate": "https://funding.communitybridge.org/projects/stackstorm",
        "News/Blog": "https://stackstorm.com/blog",
        "Security": "https://docs.stackstorm.com/latest/security.html",
        "Bug Reports": "https://github.com/StackStorm/st2/issues",
    },
)



# Copyright 2020 The StackStorm Authors.
# Copyright 2019 Extreme Networks, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import absolute_import

import os
import struct
import subprocess
import sys

from st2client.utils.color import format_status

DEFAULT_TERMINAL_SIZE_COLUMNS = 150

__all__ = ["DEFAULT_TERMINAL_SIZE_COLUMNS", "get_terminal_size_columns"]


def get_terminal_size_columns(default=DEFAULT_TERMINAL_SIZE_COLUMNS):
    """
    Try to retrieve COLUMNS value of terminal size using various system specific approaches.

    If terminal size can't be retrieved, default value is returned.

    NOTE 1: COLUMNS environment variable is checked first, if the value is not set / available,
            other methods are tried.

    :rtype: ``int``
    :return: columns
    """
    # 1. Try COLUMNS environment variable first like in upstream Python 3 method -
    # https://github.com/python/cpython/blob/master/Lib/shutil.py#L1203
    # This way it's consistent with upstream implementation. In the past, our implementation
    # checked those variables at the end as a fall back.
    try:
        columns = os.environ["COLUMNS"]
        return int(columns)
    except (KeyError, ValueError):
        pass

    def ioctl_GWINSZ(fd):
        import fcntl
        import termios

        # Return a tuple (lines, columns)
        return struct.unpack("hh", fcntl.ioctl(fd, termios.TIOCGWINSZ, "1234"))

    # 2. try stdin, stdout, stderr
    for fd in (0, 1, 2):
        try:
            return ioctl_GWINSZ(fd)[1]
        except Exception:
            sys.stderr.write("\n")

    # 3. try os.ctermid()
    try:
        fd = os.open(os.ctermid(), os.O_RDONLY)
        try:
            return ioctl_GWINSZ(fd)[1]
        finally:
            os.close(fd)
    except Exception:
        sys.stderr.write("\n")

    # 4. try `stty size`
    try:
        process = subprocess.Popen(
            ["stty", "size"],
            shell=False,
            stdout=subprocess.PIPE,
            stderr=open(os.devnull, "w"),
        )
        result = process.communicate()
        if process.returncode == 0:
            return tuple(int(x) for x in result[0].split())[1]
    except Exception:
        sys.stderr.write("\n")

    # 5. return default fallback value
    return default


class TaskIndicator(object):
    def __enter__(self):
        self.dirty = False
        return self

    def __exit__(self, type, value, traceback):
        return self.close()

    def add_stage(self, status, name):
        self._write("\t[{:^20}] {}".format(format_status(status), name))

    def update_stage(self, status, name):
        self._write("\t[{:^20}] {}".format(format_status(status), name), override=True)

    def finish_stage(self, status, name):
        self._write("\t[{:^20}] {}".format(format_status(status), name), override=True)

    def close(self):
        if self.dirty:
            self._write("\n")

    def _write(self, string, override=False):
        if override:
            sys.stdout.write("\r")
        else:
            sys.stdout.write("\n")

        sys.stdout.write(string)
        sys.stdout.flush()

        self.dirty = True



# Copyright 2020 The StackStorm Authors.
# Copyright 2019 Extreme Networks, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import absolute_import

import logging

__all__ = [
    "LogLevelFilter",
    "set_log_level_for_all_handlers",
    "set_log_level_for_all_loggers",
]


class LogLevelFilter(logging.Filter):
    """
    Filter which excludes log messages which match the provided log levels.
    """

    def __init__(self, log_levels):
        self._log_levels = log_levels

    def filter(self, record):
        level = record.levelno
        if level in self._log_levels:
            return False

        return True


def set_log_level_for_all_handlers(logger, level=logging.DEBUG):
    """
    Set a log level for all the handlers on the provided logger.
    """
    logger.setLevel(level)

    handlers = logger.handlers
    for handler in handlers:
        handler.setLevel(level)

    return logger


def set_log_level_for_all_loggers(level=logging.DEBUG):
    """
    Set a log level for all the loggers and handlers to the provided level.
    """
    root_logger = logging.getLogger()
    loggers = list(logging.Logger.manager.loggerDict.values())
    loggers += [root_logger]

    for logger in loggers:
        if not isinstance(logger, logging.Logger):
            continue

        set_log_level_for_all_handlers(logger=logger)



# Copyright 2020 The StackStorm Authors.
# Copyright 2019 Extreme Networks, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Module for parsing CLI config file.
"""

from __future__ import absolute_import

import logging
import os

from collections import defaultdict

import io

import six
from six.moves.configparser import ConfigParser


__all__ = [
    "CLIConfigParser",
    "ST2_CONFIG_DIRECTORY",
    "ST2_CONFIG_PATH",
    "CONFIG_DEFAULT_VALUES",
]

ST2_CONFIG_DIRECTORY = "~/.st2"
ST2_CONFIG_DIRECTORY = os.path.abspath(os.path.expanduser(ST2_CONFIG_DIRECTORY))

ST2_CONFIG_PATH = os.path.abspath(os.path.join(ST2_CONFIG_DIRECTORY, "config"))

CONFIG_FILE_OPTIONS = {
    "general": {
        "base_url": {"type": "string", "default": None},
        "api_version": {"type": "string", "default": None},
        "cacert": {"type": "string", "default": None},
        "silence_ssl_warnings": {"type": "bool", "default": False},
        "silence_schema_output": {"type": "bool", "default": True},
    },
    "cli": {
        "debug": {"type": "bool", "default": False},
        "cache_token": {"type": "boolean", "default": True},
        "timezone": {"type": "string", "default": "UTC"},
    },
    "credentials": {
        "username": {"type": "string", "default": None},
        "password": {"type": "string", "default": None},
        "api_key": {"type": "string", "default": None},
        "basic_auth": {
            # Basic auth credentials in username:password notation
            "type": "string",
            "default": None,
        },
    },
    "api": {"url": {"type": "string", "default": None}},
    "auth": {"url": {"type": "string", "default": None}},
    "stream": {"url": {"type": "string", "default": None}},
}

CONFIG_DEFAULT_VALUES = {}

for section, keys in six.iteritems(CONFIG_FILE_OPTIONS):
    CONFIG_DEFAULT_VALUES[section] = {}

    for key, options in six.iteritems(keys):
        default_value = options["default"]
        CONFIG_DEFAULT_VALUES[section][key] = default_value


class CLIConfigParser(object):
    def __init__(
        self,
        config_file_path,
        validate_config_exists=True,
        validate_config_permissions=True,
        log=None,
    ):
        if validate_config_exists and not os.path.isfile(config_file_path):
            raise ValueError('Config file "%s" doesn\'t exist')

        if log is None:
            log = logging.getLogger(__name__)
            logging.basicConfig()

        self.config_file_path = config_file_path
        self.validate_config_permissions = validate_config_permissions
        self.LOG = log

    def parse(self):
        """
        Parse the config and return a dict with the parsed values.

        :rtype: ``dict``
        """
        result = defaultdict(dict)

        if not os.path.isfile(self.config_file_path):
            # Config doesn't exist, return the default values
            return CONFIG_DEFAULT_VALUES

        config_dir_path = os.path.dirname(self.config_file_path)

        if self.validate_config_permissions:
            # Make sure the directory permissions == 0o770
            if bool(os.stat(config_dir_path).st_mode & 0o7):
                self.LOG.warn(
                    "The StackStorm configuration directory permissions are "
                    "insecure (too permissive): others have access."
                )

            # Make sure the setgid bit is set on the directory
            if not bool(os.stat(config_dir_path).st_mode & 0o2000):
                self.LOG.info(
                    "The SGID bit is not set on the StackStorm configuration "
                    "directory."
                )

            # Make sure the file permissions == 0o660
            if bool(os.stat(self.config_file_path).st_mode & 0o7):
                self.LOG.warn(
                    "The StackStorm configuration file permissions are "
                    "insecure: others have access."
                )

        config = ConfigParser()
        with io.open(self.config_file_path, "r", encoding="utf8") as fp:
            config.readfp(fp)

        for section, keys in six.iteritems(CONFIG_FILE_OPTIONS):
            for key, options in six.iteritems(keys):
                key_type = options["type"]
                key_default_value = options["default"]

                if config.has_option(section, key):
                    if key_type in ["str", "string"]:
                        get_func = config.get
                    elif key_type in ["int", "integer"]:
                        get_func = config.getint
                    elif key_type in ["float"]:
                        get_func = config.getfloat
                    elif key_type in ["bool", "boolean"]:
                        get_func = config.getboolean
                    else:
                        msg = 'Invalid type "%s" for option "%s"' % (key_type, key)
                        raise ValueError(msg)

                    value = get_func(section, key, raw=True)
                    result[section][key] = value
                else:
                    result[section][key] = key_default_value

        return dict(result)



# Copyright 2020 The StackStorm Authors.
# Copyright 2019 Extreme Networks, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import absolute_import
import os
import json
import mock
import tempfile
import requests
import logging

from tests import base
from st2client import shell


LOG = logging.getLogger(__name__)

USERNAME = "stanley"
PASSWORD = "ShhhDontTell"
HEADERS = {"content-type": "application/json"}
AUTH_URL = "https://127.0.0.1:9100/tokens"
GET_RULES_URL = (
    "http://127.0.0.1:9101/v1/rules/"
    "?include_attributes=ref,pack,description,enabled&limit=50"
)
GET_RULES_URL = GET_RULES_URL.replace(",", "%2C")


class TestHttps(base.BaseCLITestCase):
    def __init__(self, *args, **kwargs):
        super(TestHttps, self).__init__(*args, **kwargs)
        self.shell = shell.Shell()

    def setUp(self):
        super(TestHttps, self).setUp()

        # Setup environment.
        os.environ["ST2_BASE_URL"] = "http://127.0.0.1"
        os.environ["ST2_AUTH_URL"] = "https://127.0.0.1:9100"

        if "ST2_CACERT" in os.environ:
            del os.environ["ST2_CACERT"]

        # Create a temp file to mock a cert file.
        self.cacert_fd, self.cacert_path = tempfile.mkstemp()

    def tearDown(self):
        super(TestHttps, self).tearDown()

        # Clean up environment.
        if "ST2_CACERT" in os.environ:
            del os.environ["ST2_CACERT"]
        if "ST2_BASE_URL" in os.environ:
            del os.environ["ST2_BASE_URL"]

        # Clean up temp files.
        os.close(self.cacert_fd)
        os.unlink(self.cacert_path)

    @mock.patch.object(
        requests,
        "post",
        mock.MagicMock(return_value=base.FakeResponse(json.dumps({}), 200, "OK")),
    )
    def test_decorate_https_without_cacert(self):
        self.shell.run(["auth", USERNAME, "-p", PASSWORD])
        kwargs = {"verify": False, "headers": HEADERS, "auth": (USERNAME, PASSWORD)}
        requests.post.assert_called_with(AUTH_URL, json.dumps({}), **kwargs)

    @mock.patch.object(
        requests,
        "post",
        mock.MagicMock(return_value=base.FakeResponse(json.dumps({}), 200, "OK")),
    )
    def test_decorate_https_with_cacert_from_cli(self):
        self.shell.run(["--cacert", self.cacert_path, "auth", USERNAME, "-p", PASSWORD])
        kwargs = {
            "verify": self.cacert_path,
            "headers": HEADERS,
            "auth": (USERNAME, PASSWORD),
        }
        requests.post.assert_called_with(AUTH_URL, json.dumps({}), **kwargs)

    @mock.patch.object(
        requests,
        "post",
        mock.MagicMock(return_value=base.FakeResponse(json.dumps({}), 200, "OK")),
    )
    def test_decorate_https_with_cacert_from_env(self):
        os.environ["ST2_CACERT"] = self.cacert_path
        self.shell.run(["auth", USERNAME, "-p", PASSWORD])
        kwargs = {
            "verify": self.cacert_path,
            "headers": HEADERS,
            "auth": (USERNAME, PASSWORD),
        }
        requests.post.assert_called_with(AUTH_URL, json.dumps({}), **kwargs)

    @mock.patch.object(
        requests,
        "get",
        mock.MagicMock(return_value=base.FakeResponse(json.dumps([]), 200, "OK")),
    )
    def test_decorate_http_without_cacert(self):
        self.shell.run(["rule", "list"])
        requests.get.assert_called_with(GET_RULES_URL)

    @mock.patch.object(
        requests,
        "get",
        mock.MagicMock(return_value=base.FakeResponse(json.dumps({}), 200, "OK")),
    )
    def test_decorate_http_with_cacert_from_cli(self):
        self.shell.run(["--cacert", self.cacert_path, "rule", "list"])
        requests.get.assert_called_with(GET_RULES_URL)

    @mock.patch.object(
        requests,
        "get",
        mock.MagicMock(return_value=base.FakeResponse(json.dumps({}), 200, "OK")),
    )
    def test_decorate_http_with_cacert_from_env(self):
        os.environ["ST2_CACERT"] = self.cacert_path
        self.shell.run(["rule", "list"])
        requests.get.assert_called_with(GET_RULES_URL)



# Copyright 2020 The StackStorm Authors.
# Copyright 2019 Extreme Networks, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import absolute_import

from st2client.models.timer import Timer
from st2client.commands import resource


class TimerBranch(resource.ResourceBranch):
    def __init__(self, description, app, subparsers, parent_parser=None):
        super(TimerBranch, self).__init__(
            Timer,
            description,
            app,
            subparsers,
            parent_parser=parent_parser,
            read_only=True,
            commands={"list": TimerListCommand, "get": TimerGetCommand},
        )


class TimerListCommand(resource.ResourceListCommand):
    display_attributes = ["id", "uid", "pack", "name", "type", "parameters"]

    def __init__(self, resource, *args, **kwargs):
        super(TimerListCommand, self).__init__(resource, *args, **kwargs)

        self.parser.add_argument(
            "-ty",
            "--timer-type",
            type=str,
            dest="timer_type",
            help=(
                "List %s type, example: 'core.st2.IntervalTimer', \
                                       'core.st2.DateTimer', 'core.st2.CronTimer'."
                % resource.get_plural_display_name().lower()
            ),
            required=False,
        )

    @resource.add_auth_token_to_kwargs_from_cli
    def run(self, args, **kwargs):
        if args.timer_type:
            kwargs["timer_type"] = args.timer_type

        if kwargs:
            return self.manager.query(**kwargs)
        else:
            return self.manager.get_all(**kwargs)


class TimerGetCommand(resource.ResourceGetCommand):
    display_attributes = ["all"]
    attribute_display_order = ["type", "pack", "name", "description", "parameters"]
